﻿

# -*- coding: UTF-8 -*-
#appModules/skype.py
#A part of NonVisual Desktop Access (NVDA)
#Copyright (C) 2007-2013 Peter Vágner, NV Access Limited

#some extra features added by Williams Cuevas Herrera <zarosoft@gmail.com>
#This file is covered by the GNU General Public License.
#See the file COPYING for more details.

from comtypes import COMError
from configobj import ConfigObj
import addonHandler
import api
import wx
from threading import Thread
import gui
import nvwave
import scriptHandler
import ctypes
import os
from keyboardHandler import KeyboardInputGesture
import appModuleHandler
import controlTypes
import winUser
import NVDAObjects.IAccessible
import oleacc
import ui
from time import sleep
import windowUtils
import languageHandler
import displayModel
import queueHandler
import config
import globalVars
from logHandler import log

#initialize translation
addonHandler.initTranslation()

#constants
_addonDir = os.path.join(os.path.dirname(__file__), "..").decode("mbcs") # The root of an addon folder
_docFileName = "readme.txt" # The name of an addon documentation file
_msgNotInConversationWindow = _('Not in conversation window')
_msgContacts = _('Contacts')
_scFrom = _('From ')

def clickObject (location, rightMouseButton=False):
	(left, top, width, height) = (int(location[0]), int(location[1]), int(location[2]), int (location[3]))
		# Executa um click com o mouse num objeto
		# Se rightMouseButton = True, clica com o botão direito, caso contrário clica com o esquerdo.
	if (left+top+width+height)>0:
		x=left+(width)/2
		y=top+(height)/2
		winUser.setCursorPos (x,y)
		if rightMouseButton:
			winUser.mouse_event(winUser.MOUSEEVENTF_RIGHTDOWN,0,0,None,None)
			winUser.mouse_event (winUser.MOUSEEVENTF_RIGHTUP,0,0,None,None)
		else:
			winUser.mouse_event(winUser.MOUSEEVENTF_LEFTDOWN,0,0,None,None)
			winUser.mouse_event (winUser.MOUSEEVENTF_LEFTUP,0,0,None,None)

class TWidgetControl(NVDAObjects.IAccessible.IAccessible):
	def startMonitoring(self):
		displayModel.requestTextChangeNotifications(self, True)

	def stopMonitoring(self):
		displayModel.requestTextChangeNotifications(self, False)

	def event_textChange(self):
		queueHandler.queueFunction(queueHandler.eventQueue, self.writing)

	def writing(self):
		waveFile=os.path.join(os.path.dirname(__file__),'SkypeTyping1.wav') 
		ia=self.IAccessibleObject
		if ia.accName(1):
			if  AppModule(None).typingAnnounce:
				ui.message(ia.accname(1))
			if AppModule(None).typingSounder:
				th=Thread (target = AppModule(None).playTicks, args = ())
				th.start()
				#while ia.accName(1): #for x in range (2):#while ia.accName(1): #for x in range(6):
					#nvwave.playWaveFile(waveFile)
					#sleep(0.3)

class ChatOutputList(NVDAObjects.IAccessible.IAccessible):

	def startMonitoring(self):
		self.oldLastMessageText = None
		self.oldSecondLastMessageText = None
		self.update(initial=True)
		self._staticTextDialog=None
		displayModel.requestTextChangeNotifications(self, True)

	def stopMonitoring(self):
		displayModel.requestTextChangeNotifications(self, False)

	def reportMessage(self, text, extraData=True, specialInfo='', keyRepeatCount=0, index=0, contactName=None):
		# anuncia as mensagens automáticas do histórico ou as requesitadas com alt mais números.
		if text.startswith("["):
			# Remove the timestamp.
			text = text.split("] ", 1)[1]
		#implements now the strip extra data
		# remove o prefixo 'De' da mensagem.
		if text.index(_scFrom)==0:
			text = text[len(_scFrom):]
		#some people don't want know the time in the new skype in any mesage
		if not extraData:
			l=text.split(",")
			l=l[:-2] #strip the data in the new skype
			text=",".join(l) 
		text+= specialInfo
		ui.message(text)
		# Se a tecla foi pressionada 2 vezes, exibe uma caixa de diálogo com a mensagem correspondente.
		if keyRepeatCount==1:
			title= _('Skype Add-on - Chat with %s')%(contactName)
			text=_('Message %d:\n%s\n\nPress Esc to close this message.') %(index, text) 
			AppModule(None).showStaticTextDialog(title, text)


	def update(self, initial=False):
		reportNew = not initial and config.conf["presentation"]["reportDynamicContentChanges"]

		# Ideally, we'd determine new messages based just on the change in child count,
		# but children can be inserted in the middle when messages are expanded.
		# Therefore, we have to use message text.
		ia = self.IAccessibleObject
		newMessages = []
		lastWasEdited = False
		# The list is chronological and we're looking for new messages,
		# so scan the list in reverse.
		for c in xrange (self.childCount, -1, -1):
			try:
				if ia.accRole(c) != oleacc.ROLE_SYSTEM_LISTITEM or ia.accState(c) & oleacc.STATE_SYSTEM_UNAVAILABLE:
					# Not a message.
					continue
			except COMError:
				# The child probably disappeared after we fetched childCount.
				continue
			text = ia.accName(c)

			if not text:
				continue
			if _(' Not delivered') in text:
				text=text.replace(_(' Not delivered'),'')

			if text.startswith("[] "):
				# When timestamps are disabled,
				# Skype initially prefixes outgoing messages with "[] ".
				# However, the prefix silently disappears shortly afterwards.
				# Remove it so we aren't affected by it.
				text = text[3:]
			if self.oldLastMessageText:
				if text== self.oldLastMessageText:#text.split(',')[1] == self.oldLastMessageText.split(',')[1]:
					# No more new messages.
					break
			if text == self.oldSecondLastMessageText and len(newMessages) == 1:
				# We didn't match the last message, but this is the second last message.
				# This means the last message must have been edited, so stop here.
				lastWasEdited = True
				break
			newMessages.append(text)
			if not reportNew and (self.oldLastMessageText or len(newMessages) > 1):
				# If we're not reporting new messages, we only need to go
				# far enough so that we have the second last message.
				break

		if not newMessages:
			return

		oldLast = self.oldLastMessageText
		self.oldLastMessageText = newMessages[0]
		if not lastWasEdited:
			try:
				self.oldSecondLastMessageText = newMessages[1]
			except IndexError:
				# There was only one new message,
				# so the second last is the old last.
				self.oldSecondLastMessageText = oldLast

		if not reportNew:
			return

		for text in reversed(newMessages):
			self.reportMessage(text,AppModule(None).chatMessageTimes )



	def event_textChange(self):
		# This event is called from another thread, but this needs to run in the main thread.
		queueHandler.queueFunction(queueHandler.eventQueue, self.update)

class AppModule(appModuleHandler.AppModule):

	def __init__(self, *args, **kwargs):
		super(AppModule, self).__init__(*args, **kwargs)
		self.chatWindow = None
		self.chatOutputList = None
		self._listDialog=None
		self._staticTextDialog=None
		self.SkypeController = self.ContactsController = self.ChatsController = self.NotificationsController = False
		self.th=None
		self.lastAlert=None
		self.lastKeyReadMessage = None
		self.readConfig()
		self.bindReadMessages()

	def __del__(self):
		if self._listDialog is not None:
			self._listDialog.Destroy()
		if self._staticTextDialog is not None and False:
			self._staticTextDialog.Destroy()


	def event_NVDAObject_init(self,obj):

		if isinstance(obj, NVDAObjects.IAccessible.IAccessible) and obj.event_objectID is None and controlTypes.STATE_FOCUSED in obj.states and obj.role not in (controlTypes.ROLE_POPUPMENU,controlTypes.ROLE_MENUITEM,controlTypes.ROLE_MENUBAR):
			# The window handle reported by Skype accessibles is sometimes incorrect.
			# This object is focused, so we can override with the focus window.
			obj.windowHandle=winUser.getGUIThreadInfo(None).hwndFocus
			obj.windowClassName=winUser.getClassName(obj.windowHandle)
		if obj.value and obj.windowClassName in ("TMainUserList", "TConversationList", "TInboxList", "TActiveConversationList", "TConversationsControl"):
			# The name and value both include the user's name, so kill the value to avoid doubling up.
			# The value includes the Skype name,
			# but we care more about the additional info (e.g. new event count) included in the name.
			obj.value=None
		elif isinstance(obj, NVDAObjects.IAccessible.IAccessible) and obj.IAccessibleRole == oleacc.ROLE_SYSTEM_PANE and not obj.name:
			# Prevent extraneous reporting of pane when tabbing through a conversation form.
			obj.shouldAllowIAccessibleFocusEvent = False

	def chooseNVDAObjectOverlayClasses(self, obj, clsList):
		if obj.windowClassName == "TChatContentControl" and obj.role == controlTypes.ROLE_LIST:
			clsList.insert(0, ChatOutputList)
		elif obj.windowClassName == 'TWidgetControl':
			clsList.insert(0, TWidgetControl)

	def conversationMaybeFocused(self, obj):
		if not isinstance(obj, NVDAObjects.IAccessible.IAccessible) or obj.windowClassName != "TConversationForm" or obj.IAccessibleRole != oleacc.ROLE_SYSTEM_CLIENT:
			# This isn't a Skype conversation.
			return
		# The user has entered a Skype conversation.

		if self.chatWindow:
			# Another conversation was already focused and hasn't been cleaned up yet.
			self.conversationLostFocus()

		window = obj.windowHandle
		self.chatWindow = window
		try:
			h=self.findDescendantWindow(api.getForegroundObject().windowHandle,'TWidgetControl')
			self.TWidgetControl = NVDAObjects.IAccessible.getNVDAObjectFromEvent(h,-4,0)
			self.chatOutputList = NVDAObjects.IAccessible.getNVDAObjectFromEvent(
				windowUtils.findDescendantWindow(window, className="TChatContentControl"),
				winUser.OBJID_CLIENT, 0)
			if self.chatOutputList.children[0].role == controlTypes.ROLE_LIST:
				self.chatOutputList=self.chatOutputList.children[0]
			elif self.chatOutputList.children[1].role == controlTypes.ROLE_LIST:
				self.chatOutputList=self.chatOutputList.children[1]
			elif self.chatOutputList.children[2].role == controlTypes.ROLE_LIST:
				self.chatOutputList=self.chatOutputList.children[2]

		except LookupError:
			pass
		else:
			self.chatOutputList.startMonitoring()
			self.TWidgetControl.startMonitoring()



	def event_focusEntered(self, obj, nextHandler):
		# verifica se uma janela após a mudança de conversa com control + tab ou control+shift + tab está minimizada.
		# Se sim, a restaura.
		if self.isWindowMinimized(obj.windowHandle) and obj.role == controlTypes.ROLE_WINDOW:
			# 9 = constante para restaurar a janela.
			winUser.user32.ShowWindow(obj.windowHandle, 9)
		# Move o foco  para a caixa de edição quando uma janela de conversação é aberta.
		if self.isInConversationWindow(True)>0 and api.getFocusObject().windowClassName=='TConversationForm':
			self.setFocusInChatInput()

		self.conversationMaybeFocused(obj)
		nextHandler()


	def conversationLostFocus(self):
		self.chatWindow = None
		self.TWidgetControl.stopMonitoring()
		self.chatOutputList.stopMonitoring()
		self.TWidgetControl = None
		self.chatOutputList = None
	def event_gainFocus(self, obj, nextHandler):

		if self.chatWindow and not winUser.isDescendantWindow(self.chatWindow, obj.windowHandle):
			self.conversationLostFocus()
		# A conversation might have its own top level window,
		# but foreground changes often trigger gainFocus instead of focusEntered.
		self.conversationMaybeFocused(obj)
		nextHandler()

	def event_appModule_gainFocus(self):
		self.readConfig()
		if self.th is None and self.notifications:
			globalVars.skypeAlerts = True
			self.th = Thread (target = self.checkAlerts, args = ())
			self.th.start()


	def event_appModule_loseFocus(self):
		if self.chatWindow:
			self.conversationLostFocus()
	
	def script_toggleExtraData(self, gesture):
		self.chatMessageTimes = int(not self.chatMessageTimes)
		if self.chatMessageTimes:
			ui.message(_("Anounce message time, On"))
		else:
			ui.message(_("Anounce message time , Off"))
		self.saveConfig(self.chatMessageTimes, self.typingAnnounce, self.typingSounder, self.notifications)
	#translators:the message when press the combination on keyboard help mode.
	script_toggleExtraData.__doc__=_("Toggles the timing anounce on messages")


	def script_readMessage(self, gesture):
		if not self.isInConversationWindow():
			ui.message(_msgNotInConversationWindow)
			return
		#retrieve the list content
		listContent=self.getListContent()
		total=listContent.accChildCount
		# elimina a última mensagem para o caso das versões do skype 6.22 e 7.
		total-=(listContent.accState(total)  & oleacc.STATE_SYSTEM_UNAVAILABLE)
		key=gesture.mainKeyName
		n = int(key)
		#reindex 0 for location
		if n == 0:
			n=10
		# Verifica se a mensagem requisitada existe.
		if n>total:
			ui.message((_('Message %d not found') %n))
			return
		#now get the object and extract the message
		index = total-n+1
		message=listContent.accName(index)
		# busca por mensagens nos objetos filhos, tais como transferências de arquivos.
		specialInfo=self.getSpecialInfo(listContent, index)
		keyRepeatCount = scriptHandler.getLastScriptRepeatCount() * (self.lastKeyReadMessage == key)
		#read finally the message
		self.lastKeyReadMessage = key
		self.chatOutputList.reportMessage(message, self.chatMessageTimes, specialInfo, keyRepeatCount, index, api.getForegroundObject().name)
	script_readMessage.__doc__ = _('Read the messages by pressing Alt + numbers, 1 is the latest')

	def getSpecialInfo(self, ia, index):
		# procura nos objetos filhos por mais informações.
		# só são considerados os textos estáticos.
		specialInfo=''
		oChild=ia.accChild(index)
		totalChild = oChild.accChildCount
		if 0<totalChild<3:
			oChild=oChild.accChild(totalChild)
		if oChild:
			for x in range(1, oChild.accChildCount+1):
				if oChild.accRole(x)==oleacc.ROLE_SYSTEM_PANE:
					for j in range (oChild.accChild(x).accChildCount+1):
						if oChild.accChild(x).accRole(j)==oleacc.ROLE_SYSTEM_STATICTEXT:
							specialInfo+='\n' + oChild.accchild(x).accname(j)
				elif oChild.accRole(x)==oleacc.ROLE_SYSTEM_STATICTEXT :
					specialInfo+='\n' + oChild.accname(x)
		return specialInfo

	def getListContent(self):
		# retorna o objeto referente as mensagens do histórico.
		h=self.findDescendantWindow(api.getForegroundObject().windowHandle,'TChatContentControl')
		listContent =oleacc.AccessibleObjectFromWindow(h,-4)
		# Dentre os objetos filhos, procura o tipo lista.
		# O objeto muda dependendo da versão do skype utilizada.
		if listContent.accRole(1)==oleacc.ROLE_SYSTEM_LIST:	
			listContent=listContent.accChild(1)
		elif listContent.accRole(2)==oleacc.ROLE_SYSTEM_LIST:	
			listContent=listContent.accChild(2)
		elif listContent.accRole(3)==oleacc.ROLE_SYSTEM_LIST:	
			listContent=listContent.accChild(3)
		else:
			return None
		return listContent
	def findDescendantWindow (self, h, c):
		# retorna o handle de uma janela com uma classe especificada.
		# h, é a janela de partida da busca.
		# Retorna falso se a janela não for encontrada.
		try:
			return windowUtils.findDescendantWindow (h, className=c)
		except LookupError:
			return False

	def getFilesInHistory(self,action, fileNamesList, indexesList):
	#Retorna uma lista com os arquivos para salvar ou cancelar no histórico da conversação ativa.
		# Action, é o texto a ser procurado nos botões. (salvar ou cancelar.)
		# indexLists, recebe os índices dos objetos, considerando a hierarquia.
		# Se ouver mais de um arquivo na mesma mensagem, indexLists receberá os dados referentes aos botões salvar tudo ou cancelar tudo.
		ret=False
		ui.message (_('Wait...'))
		o = self.getListContent()
		x=o.accchildcount - 1
		while  x>0: 
			totalChild =o.accchild(x).accchildcount
			if totalChild:
			# se o objeto tiver menos de 3 filhos, faz a busca no último.
				if totalChild<3:
					oChild=o.accchild(x).accchild(totalChild)	
				else:
					oChild=o.accchild(x)
				if oChild:
					for n in range (1,oChild.accChildCount +1):
						aux=oChild.accName(n)
						if aux:
							if (aux.split(' ')[0] in action) and (oChild.accRole(n)== oleacc.ROLE_SYSTEM_PUSHBUTTON or oChild.accRole(n) == oleacc.ROLE_SYSTEM_LINK):
								if totalChild<3: # há apenas um arquivo na mensagem.
									fileNamesList.append("%s - %s" %(oChild.accName(1), oChild.accname(2)))
								else: # Há mais de um arquivo na mesma mensagem.
									totalChild=0
									aux=_('Files: ')
									for j in range (1, oChild.accChildcount+1): 
										if oChild.accRole(j) == oleacc.ROLE_SYSTEM_PANE:
											aux+= '%s - %s' %(oChild.accChild(j).accName(1), oChild.accChild(j).accName(2)) + ', '
									fileNamesList.append(aux)
								indexesList.append("%s,%s,%s" %(x, totalChild, n)) 
								ret = True
			x-=1
		return ret

	def isInConversationWindow(self, isLive=False):
		# isLive = True, Verifica se a janela é de uma chamada ativa. Se sim, retorna -1.
		import oleacc
		import winUser
		try:
			h = windowUtils.findDescendantWindow(api.getForegroundObject().windowHandle, className=		"TChatContentControl")
			if not winUser.isWindowVisible(h) and isLive:
				return -1
			o =oleacc.AccessibleObjectFromWindow (h,-4)
			if o:
				return True
			else:
				return False
		except LookupError:
			return False


	def getChatParticipants(self):
		# Retorna os nomes dos participantes de uma conversa.
		ia=None
		h= self.findDescendantWindow(api.getForegroundObject().windowHandle, "TConversationHeader")
		if h:
			ia = NVDAObjects.IAccessible.getNVDAObjectFromEvent(h,-4,0).IAccessibleObject
			x=1
			while x <= ia.accChildCount and ia.accRole(x) != oleacc.ROLE_SYSTEM_STATICTEXT:
				x+=1
			return ia.accName(x).split(", ")
		else:
			return []

	def script_saveOrCancelFile(self, gesture):
		# salva ou cancela arquivos na janela de conversa ativa.
		# só é possível cancelar os arquivos enviados pelo usuário.

		# verifica se está na janela de chamada. Nessa condição o histórico fica invisível.
		n =self.isInConversationWindow(True)
		if n <0:
			ui.message (_('You are in a live call, open a chat'))
			return
		elif n==0:
			ui.message (_msgNotInConversationWindow)
			return
		key=gesture.mainKeyName
		if key == 's': # save files.
			action = [_('Save'),_('Download')]
			msg = _('There are no files to save')
			title=_('Save files')
			label= _('Select a file to save')
		else: # cancel files
			action = _('Cancel')
			msg = _('There are no files to cancel')
			title=_('Cancel files')
			label= _('Select a file to cancel')
		fileNamesList=[]
		indexesList=[]
		ia=self.getListContent()
		if not self.getFilesInHistory(action,fileNamesList,indexesList):
			ui.message (msg)
			return
		self.showEmailsLinksDialog(1, fileNamesList, title, label, 0, ia, indexesList)
	script_saveOrCancelFile.__doc__ = _('Save or Cancel the pending files in the history')

	def listEmailsLinks (self, type, list) :
		# monta a lista de emails ou links existentes numa janela de conversas
		# retornando a mesma como parâmetro
		ui.message (_('Wait...'))
		ret=False
		o=self.getListContent()
		import re
		patternLink='(((https?|ftp|file)://)|(www))[-a-zA-z0-9+&@#/%?=~_|!:,.;]*[-a-zA-Z0-9+&@#/%=~_|]'
		patternEmail='[-a-zA-Z0-9]*@[-.a-zA-Z0-9]+(\\.[-.a-zA-Z0-9]+)*\\.(com|edu|info|gov|int|mil|net|org|biz|name|museum|coop|aero|pro|tv|[a-zA-Z]{2})'
		total = o.accChildCount
		for n in xrange (total, -1, -1):
			x=o.accChild(n).accChildCount
			while x>-1:
				if  o.accChild(n):
					aux=o.accChild(n).accName(x)
					if aux:
						rc1=re.compile(patternLink)
						rc2=re.compile(patternEmail)
						if ((not type and  re.match(rc1,aux)) or (type and re.match(rc2,aux))) and not aux in list:
							list.append(aux)
							ret=True
				x-=1
		return ret

	def showStaticTextDialog (self, title, text):
		# Abre uma  caixa de diálogo com uma mensagem requesitada.
		if self._staticTextDialog is None:
			self._staticTextDialog=StaticTextDialog(gui.mainFrame, title, text)
		if not self._staticTextDialog.IsShown():
			gui.mainFrame.prePopup()
			self._staticTextDialog.Centre()
			self._staticTextDialog.Show()
			gui.mainFrame.postPopup()



	def showEmailsLinksDialog (self, op, list, title, label,selectItem=0,ia=None,  indexesList=None):
		# exibe uma caixa de lista com um determinado conteúdo.
		# op=0, lista de links ou e-mails
		#op=1, lista de arquivos para salvar ou cancelar.
		# op=2, lista de participantes de uma chamada.
		# op=3, lista de chats abertos
		# op = 4, lista com caixa de verificações para configurações do add-on.
		if self._listDialog is not None:
			self._listDialog.Destroy() 
		self._listDialog=ListDialog (gui.mainFrame,list, title,label, op, ia, indexesList)
		self._listDialog.updateList(list,selectItem)
		if not self._listDialog.IsShown():
			gui.mainFrame.prePopup()
			self._listDialog.Centre()
			self._listDialog.Show()
			gui.mainFrame.postPopup()


	def script_addToConference(self, gesture):
		# Adiciona uma chamada a outra já existente.
		# Só funciona quando se está numa chamada simples, ou numa conferência, caso o usuário seja o administrador da mesma.
		msgAddConference=_(u'Add to group call')
		h=ctypes.windll.user32.FindWindowExA(0,0,'TCallNotificationWindow', None)
		if h!=0:
			o=NVDAObjects.IAccessible.getNVDAObjectFromEvent(h,-4,0)
			for child in o.children:
				if child.name==msgAddConference:
					clickObject (child.location)
					ui.message(_('Added to the conference'))
					return
			ui.message(_('Could not be added to the conference'))
		else:
			ui.message(_('Notify window not found'))
	script_addToConference.__doc__ = _('Adds a call to another existing')


	def script_listEmailsOrLinks(self,gesture):
		# procura por e-mails ou links numa janela de conversa.
		# caso encontre, exibe uma lista com os mesmos com as seguintes opções:
		# links: abrir o link com o navegador padrão ou enviar o link para a área de transferência.
		# E-mails: Abrir o cliente de e-mail padrão do sistema para enviar uma mensagem ou copiar o link para a área de transferência.
		if not self.isInConversationWindow():
			ui.message(_msgNotInConversationWindow)
			return
		key=gesture.mainKeyName

		op= (key!='f7')
		# op=0, busca por links.
		# op=1, busca por e-mails.
		list=[]
		ret = self.listEmailsLinks (op, list)
		if ret==0:
			ui.message([_('There are no links in history'), _('There are no E-mails in history')][op])
		elif ret>0:
			if not op: # links
				title=_("Links list")
				label=_('Select a link to open or  to copy to clipboard')
			else: # e-mails
				title=_("E-mails list")
				label=_('Select a e-mail to send a message or  to copy to clipboard')
			self.showEmailsLinksDialog(0, list, title, label)
	script_listEmailsOrLinks.__doc__ = _('F7, lists the links on history ; F9, list the emails')
	def script_addSkypeContact (self, gesture):
		# abre uma caixa de diálogo para entrar com o nome skype de um contato a ser adicionado.
		# a adição é feita fazendo uso da linha de comando do windows.
		dlg = wx.TextEntryDialog(gui.mainFrame,_("Enter contact skype ID"), _("Add new contact"))
		def callback (result):
			if  result == wx.ID_OK:
				if dlg.GetValue() != '':
					command = 'skype:%s?add' %(dlg.GetValue())
					os.startfile(command)

		gui.runScriptModalDialog(dlg, callback)
	script_addSkypeContact.__doc__ = _('Opens a dialog box to enter the Skype ID a contact to be added')

	def script_sayMicrophoneStatus(self, gesture):
		# informa o status do microfone das chamadas ativas.
		l=[_('Unmute'), _('Mute')]
		list=[]
		ret = False
		obj=api.getForegroundObject().parent
		# procura pelas janelas  de chamadas ativas.
		while obj:
			hLive=self.findDescendantWindow (obj.windowHandle, 'TLiveConversation')
			if hLive:
				list.append (hLive)
			obj=obj.next
		totalLiveCalls=len(list)
		if not totalLiveCalls:
			ui.message (_('Live conversation  window not found'))
			return
		for x in range (totalLiveCalls):
			hLive=list[x]
			o=oleacc.AccessibleObjectFromWindow(hLive,-4)
			total = o.accChildCount
			# procura pelo objeto filho relacionado a informações da chamada.
			childId=False
			while total:
				if o.accName(total)=="CallToolbar":
					childId=total
					break
				total-=1

			# procura pelo objeto filho relativo ao microfone.
			if childId:
				o=o.accchild(childId)
				for j in range (1, o.accChildCount+1):
					name=o.accname(j)
					if name in l:
						iMic=l.index(name)
						iMic=not(iMic) 
						msg=_('Live call with %s, the microphone is %s')
						ui.message (msg %(o.accParent.accparent.accparent.accName(0), [_('enabled'), _('disabled')][iMic]))
						ret = True
			if not ret:
				ui.message (_('Microphone Object not found'))
	script_sayMicrophoneStatus.__doc__ = _('Reports the status of the microphones of active calls')

	def script_sayStatus(self, gesture):
		globalVars.marcos=((scriptHandler._lastScriptRef))
		# anuncia o status do usuário.
		# A janela principal do skype deve estar visível para que funcione.
		hMain=winUser.user32.FindWindowA ('tSkMainForm', None)
		if not hMain:
			ui.message (_('The main Skype window was not found'))
			return
		hSelfControl=self.findDescendantWindow (hMain,'TMyselfControl')
		if not hSelfControl:
			ui.message(_('The user information object was not found'))
			return
		o = oleacc.AccessibleObjectFromWindow(hSelfControl,-4)
		childId=0
		count = o.accChildCount
		while count>0 and childId==0 :
			childId=count*(o.accRole(count)==oleacc.ROLE_SYSTEM_BUTTONDROPDOWN)
			count-=1
		if not childId:
			ui.message(_('Status buttonDrop down not found'))
			return
		status= o.accName(childId).split(',')[0]
		ui.message(_('You are %s') %status)
	script_sayStatus.__doc__ = _("Announces the user status")

	def script_moveToChat(self,gesture):
		# Alterna entre as janelas de conversa  ao se usar control+tab e  control+shift+tab.
		hForeground = api.getForegroundObject().windowHandle
		key="+".join(gesture.modifierNames)+"+"+gesture.mainKeyName
		if not 'tab' in key:
			n=2
		else:
			n='control+tab' in key
		lChats = self.getActiveChatsList()
		totalChats=len(lChats)
		if not totalChats:
			ui.message (_("Chats window not found"))
			return
		lChats.sort()
		isInChat = hForeground in lChats
		if isInChat and totalChats==1: # já está numa janela de conversa e esta é a única ativa.
			ui.message (_('This is the only open chat window'))
			return
		if n==2: # exibe uma lista com as janelas de conversas ativas.
			# determinando o item que será selecionado por padrão na lista.
			if isInChat:
				selectItem = lChats.index(hForeground)
			else:
				selectItem=0
			# converte os handles da lista em strings.
			lChats=[str(x) for x in lChats]
			title=_("List of active chats")
			label=_('Select a chat to open ')
			#chama a função que exibe a lista.
			self.showEmailsLinksDialog(3,lChats, title, label, selectItem) 
		else: # mudanças com control tab e control shift tab.
			if isInChat:
				# obtém o índce da janela ativa na lista.
				index=lChats.index(hForeground)
			else: 
				# Não está numa janela de conversa.
				# se pressionado control tab, move o foco para a primeira janela da lista Index assumirá -1.
				# se pressionado control+shift+tab, move o foco para a última janela da lista. Index assumirá 0.
				index=0-n
			if n: # vai para a próxima janela.
				index= index+1-totalChats*(index==totalChats-1)
			else: # vai para a janela anterior.
				index=index-1+totalChats*(index==0)
			h=lChats[index]
			obj=NVDAObjects.IAccessible.getNVDAObjectFromEvent(h,-4,0)
			# move o foco para a janela de chat.
			obj.setFocus()
	script_moveToChat.__doc__ = _('Moves the focus between the open conversations windows or displays them in a list')

	def setFocusInChatInput(self):
		# Move o foco para a caixa de edição de uma janela de conversa usando SendMessage.
# 40 = WM_NEXTDLGCTL 
		h=self.findDescendantWindow(api.getForegroundObject().windowHandle,'TChatRichEdit')
		winUser.sendMessage(api.getForegroundObject().windowHandle,40,h,1)

	def script_switchToRequestedObjects(self, gesture):
		# move o foco para alguns objetos nas janelas de conversa/janela principal.
		key=int(gesture.mainKeyName)
		# key=1, contact list
		# key=2, recents list
		# key=3, live calls list
		# key=4, chat input
		# key=5, history
		n = self.isInConversationWindow(True)
		if key<=2: # contact list or ercents list.
			keystroke=KeyboardInputGesture.fromName("alt+%s" %key)
			sleep(0.7)
			keystroke.send()
		elif key==4 or key==5: 
			if n<0: 
				ui.message ([_('The chat input is not visible'), _('The history is not visible')][key==5])
				return
			elif not n:
				ui.message (_msgNotInConversationWindow)
				return
			if key==4:
				# set focus in input editable text using sendMessage.
				self.setFocusInChatInput()
			else:
				# move focus to history
				if n<0: 
					ui.message (_('The history is not visible'))
					return
				self.getListContent().accselect(1,0)
		else: # live calls list.
			# obtém o objeto lista de chamadas ativas na janela principal.
			o = self.getTconversationControlsObject (1)
			if not o:
				ui.message (_('Active calls list not found'))
				return
			o.accSelect(1,0)
	script_switchToRequestedObjects.__doc__ = _('Moves focus to specific areas of the screen with ctrl + numbers: 1, contact list ; 2, recents messages list ; 3, list of active calls ; 4, chat input box ; 5, history.')

	def script_openContacOptions(self, gesture):
		# Exibe uma  lista dos nomes dos participantes de uma chamada
		# Ao selecionar um, pode-se abrir o menu de contexto  do contato.
		h=self.findDescendantWindow(api.getForegroundObject().windowHandle,"TLiveConversation" )
		if not h:
			ui.message (_("You aren't in live call window"))
			return
		o = oleacc.AccessibleObjectFromWindow(h,-4)

		n=1
		participantsList=[]
		indexesList=[]
		# procura pelos painéis com os nomes dos participantes.
		while n<=o.accChildCount:
			name=o.accName(n)
			if o.accRole(n) == oleacc.ROLE_SYSTEM_PANE and name!='CallToolbar':
				# adiciona numa lista o nome dos participantes e noutra os índices dos objetos filhos.
				participantsList.append(name)
				indexesList.append(n)
			n+=1
		title=_('Participants of the active call')
		label=_('Select a participant to open options')
		# chama a função que exibe a lista.
		self.showEmailsLinksDialog (2, participantsList, title, label, 0, o, indexesList)
	script_openContacOptions.__doc__ = _('Lists the participants of a conference, choosing one can be opened his context menu')

	def getLiveCall (self):
		# retorn o handle da chamada ativa, caso contrário, retorna false.
		return self.findDescendantWindow(api.getForegroundObject().windowHandle,"TLiveConversation" )

	def getCallDuration(self):
		# retorna a duração de uma chamada ativa.
		h=self.getLiveCall()
		if not h:
			return ''
		o = oleacc.AccessibleObjectFromWindow(h,-4)
		import re
		pattern=re.compile('^\d+:\d')
		duration = ''
		for x in range (1, o.accChildcount+1):
			if o.accrole(x)==oleacc.ROLE_SYSTEM_STATICTEXT and re.match (pattern,o.accName(x)):
				duration=o.accname(x)
		return duration

	def script_sayWindowTitle(self, gesture):
		# Anuncia o título da janela. Se for uma chamada, adiciona a duração da chamada.
		title=api.getForegroundObject().name
		if self.getLiveCall():
			title+=_(' - Call duration: %s')  %self.getCallDuration()
		ui.message(title)

	def bindReadMessages(self):
		# vincula os atalhos alt + números à leitura das mensagens.
		# atribui control + g para salvar arquivos para o idioma pt_PT
		for k in range(10):
			self.bindGesture("kb:alt+%i" %k, "readMessage")
		lang = languageHandler.curLang
		if lang == 'pt_PT':
			self.bindGesture("kb:control+g", "saveOrCancelFile")
		else:
			self.bindGesture("kb:control+s", "saveOrCancelFile")

	def getTconversationControlsObject(self, op):
		# returns IAccessibleObject requested in main window:
		# op=0, contact List.
		# op=1, active calls list
		name = [_msgContacts, _('Active calls')][op]
		hMain=winUser.user32.FindWindowA('TSKmainForm', None)
		h=self.findDescendantWindow(hMain,"TConversationsControl")
		if not h:
			return None
		o = oleacc.AccessibleObjectFromWindow(h,-4)
		total = o.accChildCount
		child=1
		while child<= total:
			if  o.accName(child)==name and not o.accState(child) & oleacc.STATE_SYSTEM_INVISIBLE:
				return o.accChild(child)
			child+=1
		return None

	def isWindowMinimized(hself, h):
		# retorna True se uma dada  janela estiver minimizada.
		# utilizada principalmente nas mudanças de janelas de conversa.
		obj = NVDAObjects.IAccessible.getNVDAObjectFromEvent(h,-2,0)
		for child in obj.children:
			if _('Minimize') in child.name:
				return False
		return True

	def getActiveChatsList (self):
		# retorna uma lista com todas as janelas de conversas ativas.
		oTop=api.getForegroundObject().parent
		lChats=[]
		while oTop:
			if oTop.windowClassName=='TConversationForm':
				lChats.append(oTop.windowHandle)
			oTop=oTop.next
		return lChats

	def getProcesses(self):
		from ctypes import *
		psapi = windll.psapi
		kernel = windll.kernel32
		l=[]
		arr = c_ulong * 256
		lpidProcess= arr()
		cb = sizeof(lpidProcess)
		cbNeeded = c_ulong()
		hModule = c_ulong()
		count = c_ulong()
		modname = c_buffer(30)
		PROCESS_QUERY_INFORMATION = 0x0400
		PROCESS_VM_READ = 0x0010
		psapi.EnumProcesses(byref(lpidProcess),cb,byref(cbNeeded))
		nReturned = cbNeeded.value/sizeof(c_ulong())
		pidProcess = [i for i in lpidProcess][:nReturned]
		for pid in pidProcess:
			hProcess = kernel.OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ,False, pid)
			if hProcess:
				psapi.EnumProcessModules(hProcess, byref(hModule), sizeof(hModule), byref(count))
				psapi.GetModuleBaseNameA(hProcess, hModule.value, modname, sizeof(modname))
				l.append ("".join([ i for i in modname if i != '\x00']))
				l.append ("".join([ i for i in modname if i != '\x00']))            #-- Clean up)
				for i in range(modname._length_):
					modname[i]='\x00'
				kernel.CloseHandle(hProcess)
		return l


	def readConfig(self):
		#Read configuration set from Skype.ini
		self.chatMessageTimes, self.typingAnnounce, self.typingSounder, self.notifications, self.alerts = 0, 1, 1, 1, []
		config_skype = os.path.join(globalVars.appArgs.configPath,"Skype.ini")
		if os.path.isfile(config_skype):
			config = ConfigObj(config_skype)
			try:
				self.chatMessageTimes = eval(config['Skype Settings']['Chat message times'])
				self.typingAnnounce = eval(config['Skype Settings']['Typing Announce'])
				self.typingSounder = eval(config['Skype Settings']['Typing Sounder'])
				self.notifications = eval(config['Skype Settings']['Notifications'])
				l = config ['Alerts list'].keys()
				l.sort()
				#self.alerts = [config['Alerts list'][str(x)].decode('latin-1') for x in l]
				self.alerts = []
				for x in l:
					try:
						self.alerts.append(config['Alerts list'][str(x)].decode('latin-1'))
					except:
						self.alerts.append(config['Alerts list'][str(x)])
			except (KeyError, IOError):
				pass #return ui.message(_("I can not load the settings!"))

	def script_config(self, gesture):
		title = _('Skype Add-on Configurations')
		label = _('Options')
		list=[[_('Chat Message Times'), self.chatMessageTimes], [_('Typing Announce'), self.typingAnnounce], [_('Typing Sounder'), self.typingSounder], [_('Announce alerts'), self.notifications]]
		self.showEmailsLinksDialog(4, list, title, label, 0)
	script_config.__doc__ = _('Opens a dialog box for the add-on configuration')

	def saveConfig(self, chatMessageTimes, typingAnnounce, typingSounder , notifications):
		# grava as alterações de configurações e alertas.
		config = ConfigObj()
		config_skype = os.path.join(globalVars.appArgs.configPath,"Skype.ini")
		config.filename = config_skype
		config ["Skype Settings"] = {"Chat message times": chatMessageTimes, "Typing Announce": typingAnnounce, "Typing Sounder": typingSounder, "Notifications": notifications}
		#config ['Alerts list'] = dict ([(str(self.alerts.index(alert)), alert.encode('latin-1')) for alert in self.alerts])
		x = 0
		temp=[]
		for alert in self.alerts:
			try:
				temp.append((str(x), alert.encode('latin-1')))
			except:
								temp.append((str(x), alert))
			x += 1
		config ['Alerts list'] =dict(temp)
		try:
			config.write()
		except IOError:
			pass

	def script_help(self, gesture):
		folder = self.getDocFolder()
		file=os.path.join(folder, _docFileName)
		if os.path.isfile(file):
			f = open(file, 'r')
			text= f.read()
			f.close()
			self.showStaticTextDialog (_('Help for Skype 7 add-on'), text)
	script_help.__doc__ = _('Displays a dialog box with information about the add-on')

	def getDocFolder(self,addonDir=_addonDir):
		langs = [languageHandler.getLanguage(), "en"]
		for lang in langs:
			docFolder = os.path.join(addonDir, "doc", lang)
			if os.path.isdir(docFolder):
				return docFolder
			if "_" in lang:
				tryLang = lang.split("_")[0]
				docFolder = os.path.join(addonDir, "doc", tryLang)
				if os.path.isdir(docFolder):
					return docFolder
				if tryLang == "en":
					break
			if lang == "en":
				break
		return None

	def playTicks (self):
		waveFile=os.path.join(os.path.dirname(__file__),'SkypeTyping1.wav') 
		fore = api.getForegroundObject()
		contactName=fore.name
		h = self.findDescendantWindow(fore.windowHandle,'TWidgetControl')
		try:
			ia = oleacc.AccessibleObjectFromWindow(h, -4)
			while ia.accName(1) and fore.name == contactName and fore.windowClassName == 'TConversationForm':
				nvwave.playWaveFile(waveFile)
				sleep(0.4)
		except COMERROR:
			pass

	def checkAlerts (self):
		from datetime import datetime
		while 'Skype.exe' in self.getProcesses() and self.notifications and globalVars.skypeAlerts:
			today=datetime.today()
			o=None
			alerts=''
			cl = "TTrayAlert" 
			cl1 = "TCallNotificationWindow"
			h = winUser.user32.FindWindowA(cl,None)
			if not h:
				h = winUser.user32.FindWindowA(cl1, None)
			if h :
				try:
					o=oleacc.AccessibleObjectFromWindow (h,-4)
					for x in range (1, o.accChildcount+1):
						if (o.accRole(x) == oleacc.ROLE_SYSTEM_STATICTEXT or o.accRole(x) == oleacc.ROLE_SYSTEM_ALERT):
							alerts += o.accName(x) + ' '
							if o.accName(0):
								break
					if alerts != '' and alerts != self.lastAlert:
						self.lastAlert = alerts
						ui.message('Skype: ' + alerts)
						self.alerts.insert(0,_('%s, Registered on %d/%d/%d at %d:%d:%d') %(alerts, today.day, today.month, today.year, today.hour, today.minute, today.second))
						self.alerts=self.alerts[:10]
					self.saveConfig(self.chatMessageTimes, self.typingAnnounce, self.typingSounder, self.notifications)
				except:
					pass
			sleep(0.3)
		self.th=None

	# início de scripts com comandos em camadas.
	# tecla de comando skype = shift+Esc
	# subcomandos: o para contatos; c, para conversas e n, para notificações.

	def finish(self):
		self.SkypeController = self.ContactsController = self.ChatsController = self.NotificationsController = False
		self.clearGestureBindings()
		self.bindReadMessages()
		self.bindGestures(self.__gestures)

	def script_error(self, gesture):
		import tones
		tones.beep(120, 100)

	def script_SkypeControllerPrefix(self, gesture):
		# primeiro nível de comandos em camadas.
		if not self.SkypeController:
			self.bindGestures(self.__SkypeControllerGestures)
			self.SkypeController = True
			ui.message(_("Skype Controller"))
		else:
			self.script_error(gesture)
			self.finish()
	# Translators: Mensagem de ajuda para a o comando de camada skype.
	script_SkypeControllerPrefix.__doc__=_("Skype Controller layer command. See add-on guide for available commands.")
	# início do sub comando contatos
	def script_contacts(self, gesture):
		if not self.ContactsController:
			self.bindGestures(self.__ContactsControllerGestures)
			ui.message (_msgContacts)
			self.ContactsController = True
		else:
			self.script_error(gesture)
			self.finish()
	script_contacts.__doc__ = _('Subcommand contact layers, has the following commands: the, a, d, n, i')

	def script_listContactsByStatus (self, gesture):
		# lista os contatos com um determinado status.
		# obtendo o objeto relativo a lista de contatos.
		o = self.getTconversationControlsObject(0)
		if not o:
			ui.message(_('Contact list object not found'))
			self.finish()
			return
		key=gesture.mainKeyName
		n = ['o', 'a', 'd', 'n'].index(key)
		statusList=[_('Online') ,_('Away'), _('Do Not Disturb'), _('This person has not shared their details with you.')]
		status= statusList[n]
		import re
		pattern= re.compile(', %s ?' %status)
		list=[]
		for x in range (1, o.accChildCount+1):
			name=o.accName(x)
			lFind = re.findall(pattern, name)
			if len(lFind):
				contactName= name.split(',')[0 + (_('Unread messages') in name) ]
				list.append(contactName)
		total=len(list)
		if total:
			ui.message ((_('Numbers of %s contacts:  %d')) %(status, total))
			msg = ','.join(list)
			ui.message(msg)
		else:
			ui.message(_('There were no contacts with the %s status') %status)
		self.finish()
	script_listContactsByStatus.__doc__ = _('List contacts with certain status:o, for online ; a, for away; d, for busy; N, for those who did not share details')

	def script_copyContactIdToClipboard(self, gesture):
		# copia para a área de transferência o nome skype de um contato da lista de contatos.
		obj=api.getFocusObject()
		if obj.windowClassName != "TConversationsControl" or obj.role != controlTypes.ROLE_LISTITEM:
			ui.message (_('You must be on a contact in list to use this resource'))
			self.finish()
			return
		name=obj.name
		# não funciona para contatos do messenger ou facebook.
		x = (_('Messenger') in name) + 2*((_('Facebook') in name)) -1
		if x>=0:
			ui.message ([_("Unavailable for messenger contacts"), _("Unavailable for Facebook contacts")][x])
			self.finish()
			return
		x =(_('Unread messages') in name)
		contactName=name.split(',')[x].strip()
		skypeId=name.split(',')[1+x].strip()
		if api.copyToClip(skypeId):
			ui.message (_('The skype ID of %s was sent to the clipboard') %contactName)
		self.finish()
	script_copyContactIdToClipboard.__doc__ = _('Copy the Skype id of a contact in the contact list to the clipboard.')
	# início do subcomando chats.

	def script_chats(self, gesture):
		if not self.ChatsController:
			self.bindGestures(self.__ChatsControllerGestures)
			ui.message (_('Chats'))
			self.ChatsController = True
		else:
			self.script_error(gesture)
			self.finish()
	script_chats.__doc__ = _('Subcommand chats layers, has the following commands: c and l')

	def script_listOrCloseChatWindows (self, gesture):
		key=gesture.mainKeyName
		# key = c,  fecha todas as janelas de conversas ativas. após fechá-las, foca a janela principal, se esta estiver visível.
		#key = l, lista as janelas de conversas ativas
		# obtendo a lista dos identificadores de janelas de conversas ativas.
		lChats = self.getActiveChatsList()
		total = len(lChats)
		if not total:
			ui.message (_("Chats window not found"))
		elif key == 'c':
			# fechando as janelas.
			for h in lChats:
				winUser.PostMessage (h, 0x10, 0, 0)
			sleep(0.3)
			# verificando se a janela principal está ativa.
			hMain = winUser.user32.FindWindowA ('TSKmainForm', None)
			if hMain:
				obj = NVDAObjects.IAccessible.getNVDAObjectFromEvent (hMain, -4, 0)
				# se o foco já não estiver sobre a janela principal, foca a mesma.
				if api.getForegroundObject() != obj:
					obj.setFocus()
		else: # lista as janelas de conversas.
			ui.message (_('Total of active chats: %d.') %total)
			for h in lChats:
				ui.message (winUser.getWindowText(h ))
		self.finish()
	script_listOrCloseChatWindows.__doc__ = _('Reports or closes all windows of active conversations. c = reports, l, list.')

	# início do subcomandos de camadas notificações.
	def script_notifications(self, gesture):
		if not self.NotificationsController:
			self.bindGestures(self.__NotificationsControllerGestures)
			ui.message (_('Notifications'))
			self.NotificationsController = True
		else:
			self.script_error(gesture)
			self.finish()
	script_notifications.__doc__ = _('Subcommand layers notifications, has the following commands: l and numbers 0 through  9')

	def script_sayAlerts (self, gesture):
		key=gesture.mainKeyName
		total = len (self.alerts)
		if key == 'l':
			if not len (self.alerts):
				ui.message (_('No alerts'))
			else:
				for alert in self.alerts:
					ui.message(alert)
			self.finish()
		else: # teclas de 1 a 0. 
			n = int(key)
			if n == 0:
				n = 10
			if n > total:
				ui.message (_('No alert %d') %n)
			else:
				ui.message (self.alerts[n-1])
	script_sayAlerts.__doc__ = _('List the last 10 alerts or specific warning deterninado by the numbers 0 through 9')

	def script_escapeNotifications (self, gesture):
		if self.NotificationsController:
			ui.message (_('Leaving notifications command'))
			self.finish()
	script_escapeNotifications.__doc__ = _('Exits the  notifications subcommand')

	# definindo os atalhos disponíveis para o comando de camadas skype.
	__SkypeControllerGestures={
		"kb:o":"contacts",
		"kb:c":"chats",
		"kb:n":"notifications",
	}

	# definindo os atalhos para o subcomando de camadas contatos.
	__ContactsControllerGestures={
		"kb:o":"listContactsByStatus",
		"kb:a":"listContactsByStatus",
		"kb:d":"listContactsByStatus",
		"kb:n":"listContactsByStatus",
		"kb:i":"copyContactIdToClipboard",
	}

	# definindo os atalhos para o subcomando de camadas chats.
	__ChatsControllerGestures={
		"kb:c":"listOrCloseChatWindows",
		"kb:l":"listOrCloseChatWindows",
	}

	#definindo os atalhos para o subcomando notificações.
	__NotificationsControllerGestures={
		"kb:l":"sayAlerts",
		"kb:escape":"escapeNotifications",
		"kb:1":"sayAlerts",
		"kb:2":"sayAlerts",
		"kb:3":"sayAlerts",
		"kb:4":"sayAlerts",
		"kb:5":"sayAlerts",
		"kb:6":"sayAlerts",
		"kb:7":"sayAlerts",
		"kb:8":"sayAlerts",
		"kb:9":"sayAlerts",
		"kb:0":"sayAlerts",
	}



	__gestures ={
		"kb:control+shift+c": "config",
		"kb:f2": "toggleExtraData",
		"kb:f5": "openContacOptions",
		"kb:f7": "listEmailsOrLinks",
		"kb:f8": "addToConference",
		"kb:f9": "listEmailsOrLinks",
		"kb:shift+escape":"SkypeControllerPrefix",
		"kb:control+k": "sayMicrophoneStatus",
		"kb:control+shift+o": "addSkypeContact",
		#"kb:control+s": "saveOrCancelFile",
		"kb:control+t": "sayStatus",
		"kb:control+w": "saveOrCancelFile",
		"kb:control+tab": "moveToChat",
		"kb:control+shift+tab": "moveToChat",
		"kb:control+l": "moveToChat",
		"kb:NVDA+h": "help",
		"kb:control+1": "switchToRequestedObjects",
		"kb:control+2": "switchToRequestedObjects",
		"kb:control+3": "switchToRequestedObjects",
		"kb:control+4": "switchToRequestedObjects",
		"kb:control+5": "switchToRequestedObjects",
		"kb:NVDA+t": "sayWindowTitle",
		}

class ListDialog(wx.Dialog):
	def __init__(self, parent, list, title, labelCaption, typeList, ia=None, indexesList = None):
		self.list = list
		self.typeList=typeList
		self.ia=ia
		self.indexesList=indexesList
		self.states=[]
		if self.typeList==4: #check list.
			self.states=([obj[1] for obj in self.list])
		super(ListDialog, self).__init__(parent, title=title)
		self.title=title
		if self.typeList==0: # lista de e-mails ou links.
			labelButton1=[_('&Send e-mail with default client'), _('&Open link with default browser')][not '@' in self.list[0]]
			labelButton2=[_('&Copy e-mail to clipboard'), _('&Copy link to clipboard')][not '@' in self.list[0]]
		else:
			labelButton1='Ok'
		# cria a interface.
		mainSizer = wx.BoxSizer(wx.VERTICAL)
		tasksSizer = wx.BoxSizer(wx.VERTICAL)
		# cria a list view.
		tasksLabel = wx.StaticText(self, -1, label=labelCaption)
		tasksSizer.Add(tasksLabel)
		self.listBox = wx.ListCtrl(self, size=(550, 250),style=wx.LC_REPORT|wx.BORDER_SUNKEN)
		tasksSizer.Add(self.listBox, proportion=8)
		#mainSizer.Add(tasksSizer)
		# cria os botões.
		buttonsSizer = wx.BoxSizer(wx.HORIZONTAL)
		button1ID=wx.NewId()
		self.button1 = wx.Button(self,button1ID, labelButton1)
		buttonsSizer.Add(self.button1)
		if self.typeList==0: # lista de e-mails ou links , exibe mais 1 botão.
			button2ID=wx.NewId()
			self.button2 = wx.Button(self, button2ID, labelButton2)
			buttonsSizer.Add(self.button2)
			# define os eventos para o segundo botão.
			wx.EVT_KILL_FOCUS (self.button2,self.onLostFocus)
			self.Bind(wx.EVT_BUTTON,self.onButton2,id=button2ID)
		self.cancelButton = wx.Button(self, wx.ID_CANCEL)
		buttonsSizer.Add(self.cancelButton)
		tasksSizer.Add(buttonsSizer)
		mainSizer.Add(tasksSizer)
		# define os eventos para os demais controles.
		wx.EVT_KILL_FOCUS (self.listBox,self.onLostFocus)
		wx.EVT_KILL_FOCUS (self.button1,self.onLostFocus)
		wx.EVT_KILL_FOCUS (self.cancelButton,self.onLostFocus)
		self.Bind(wx.EVT_BUTTON,self.onButton1,id=button1ID)
		self.listBox.Bind(wx.EVT_KEY_DOWN, self.onKeyPress)
		mainSizer.Fit(self)
		self.SetSizer(mainSizer)


	def onButton1 (self,evt):
		self.Hide()
		if not self.typeList:
			self.openLink()
		elif self.typeList==1: #files.
			self.saveOrCancelFiles()
		elif self.typeList == 2: # participants options.
			self.selectParticipant()
		elif self.typeList==3: # select a chat
			self.selectChat()
		else: # configurations, check list.
			self.checkChanges()
		evt.Skip()

	def onButton2 (self,evt):
		self.Hide()
		index=self.listBox.GetFocusedItem()
		if not self.typeList: # lista de links ou e-mails.
			item=self.listBox.GetItemText(index)
			api.copyToClip(item)
		evt.Skip()

	def onKeyPress(self, event):
		keycode = event.GetKeyCode()
		if keycode==wx.WXK_RETURN:
			self.Hide()
			if self.typeList==0: # lista de links ou e-mails.
				self.openLink()
			elif self.typeList==1: #files
				self.saveOrCancelFiles()
			elif self.typeList==2: # participants options
				self.selectParticipant()
			elif self.typeList==3: # lista de conversas ativas.
				self.selectChat()

		elif  keycode == wx.WXK_SPACE and self.listBox.GetColumnCount()==2: # check list.
			# marca ou desmarca os itens da lista.
			index=self.listBox.GetFocusedItem()
			newStatus=int(not self.list[index][1])
			self.list[index][1]=newStatus
			self.updateList(self.list,index)
		event.Skip()

	def openLink (self):
		# abre um link com o navegador padrão ou envia ui. e-mail com o cliente padrão.
		import os
		index=self.listBox.GetFocusedItem()
		item=self.listBox.GetItemText(index)
		if '@' in item:
			os.startfile('mailto:%s' %item)
		else:
			os.startfile(item)

	def saveOrCancelFiles (self):
		import time
		time.sleep (0.5)
		focus=api.getFocusObject()
		index=self.listBox.GetFocusedItem()
		aux=self.indexesList[index]
		# child1 é o objeto filho que contém a mensagem de envio do arquivo.
		child1= int(aux.split(',')[0])
		# child2 é o painel que contém os botões.
		child2= int(aux.split(',')[1])
		# child3, é o objeto relativo aos botões salvar, salvar tudo, cancelar ou cancelar tudo.
		child3= int(aux.split(',')[2])
		if focus.name != self.ia.accname(child1): # verifica se o foco já está sobre a mensagem, se não estiver, seleciona a msg.
			self.ia.accSelect(1, child1)
			self.ia.accSelect(2, child1)
		k=self.ia.accChild(child1).accChild(child2)
		# clica no objeto correspondente.
		clickObject (k.accLocation(child3))

	def selectParticipant (self):
		# clica com o botão direito do mouse no painel relativo ao contatos selecionado.
		index=self.listBox.GetFocusedItem()
		sleep(0.5)
		n=self.indexesList[index]
		self.ia.accSelect(1,n)
		clickObject(self.ia.accLocation(n), True)

	def selectChat (self):
		# move o foco para a janela de conversa selecionada na lista.
		index=self.listBox.GetFocusedItem()
		h= int(self.list[index])
		obj=NVDAObjects.IAccessible.getNVDAObjectFromEvent(h,-4,0)
		obj.setFocus()

	def checkChanges (self):
		# Se houve alteração de status, grava as mudanças no arquivo ini.
		currentStatesList = [x[1] for x in self.list]
		if self.states != currentStatesList: # houve alterações.
			AppModule(None).saveConfig (currentStatesList[0], currentStatesList[1], currentStatesList[2], currentStatesList[3])

	def onLostFocus(self,evt):
		obj = api.getForegroundObject()
		hasFocus= False
		for child in obj.children:
			hasFocus+= child.hasFocus
		if not hasFocus:
			self.Hide()
			evt.Skip()


	def updateList(self, list, index):
		self.listBox.ClearAll()
		self.listBox.InsertColumn(0,_('Item'))
		if self.typeList!=4: # não é um check list.
			for item in list:
				if  self.typeList == 3: # A lista contém os identificadores das janelas de chats.
					# adiciona à lista o título das janelas.
					item= winUser.getWindowText(int(item))
				self.listBox.Append ([item])
		else: # check list.
			self.listBox.InsertColumn(1, _('Status'))
			op=[_("Disabled"),_("Enabled")]
			for x in list:
				self.listBox.Append ([x[0],op[x[1]]])

		self.listBox.Focus(index)
		self.listBox.Select(index)

class StaticTextDialog(wx.Dialog):
	def __init__(self, parent, title, text):
		self.text = text
		self.title=title
		super(StaticTextDialog, self).__init__(parent, title=title)
		# Create interface
		mainSizer = wx.BoxSizer(wx.VERTICAL)
		tasksSizer = wx.BoxSizer(wx.VERTICAL)
		# Create a static text.
		read_only_txtCtrlId =120#wx.NewId()
		self.read_only_txtCtrl = wx.TextCtrl(self,read_only_txtCtrlId,self.text,style=wx.TE_MULTILINE|wx.TE_READONLY, size=(800,200))
		tasksSizer.Add(self.read_only_txtCtrl)
		mainSizer.Add(tasksSizer)
		# Create Close button.
		buttonsSizer = wx.BoxSizer(wx.HORIZONTAL)
		self.closeButton = wx.Button(self, wx.ID_CANCEL, _('&Close'))
		buttonsSizer.Add(self.closeButton)
		# TextCtrl and buttons events.
		wx.EVT_KILL_FOCUS (self.read_only_txtCtrl,self.onLostFocus)
		wx.EVT_KILL_FOCUS (self.closeButton,self.onLostFocus)
		wx.EVT_CHAR (self.read_only_txtCtrl,self.onKeyPress)

		tasksSizer.Add(buttonsSizer)
		mainSizer.Add(tasksSizer)
		mainSizer.Fit(self)
		self.SetSizer(mainSizer)
		self.read_only_txtCtrl.SetFocus()
		self.read_only_txtCtrl.SetInsertionPoint(0)


	def onKeyPress(self, event):
		keycode = event.GetKeyCode()
		# verifica se ctrl+A foi pressionado para copiar todo o texto.
		if keycode==1:
			self.read_only_txtCtrl.SelectAll()
		event.Skip()
	def onLostFocus(self,evt):
		obj = api.getForegroundObject()
		hasFocus= False
		for child in obj.children:
			hasFocus+= child.hasFocus
		if not hasFocus:
			self.Hide()
			evt.Skip()
